
#include "TArtAnaLoopNEBULAHPC.hh"

#include <iostream>

#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"

#include "TArtSAMURAIParameters.hh"
#include "TArtCalibNEBULA.hh"
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"
#include "TArtCalibNEBULAHPC.hh"
#include "TArtNEBULAHPC.hh"
#include "TArtNEBULAHPCPara.hh"

#include "EAnalyser.hh"
#include "EWNum.hh"

TArtAnaLoopNEBULAHPC::TArtAnaLoopNEBULAHPC()
  : TArtAnaLoop(), samuraiparameters(0), calibnebula(0), calibnebulahpc(0)
{;}

TArtAnaLoopNEBULAHPC::~TArtAnaLoopNEBULAHPC()
{
  Destruct();
}

void TArtAnaLoopNEBULAHPC::Construct()
{
  samuraiparameters = new TArtSAMURAIParameters;
  samuraiparameters->LoadParameter("db/NEBULA.xml");
  samuraiparameters->LoadParameter("db/NEBULAHPC.xml");
  calibnebula= new TArtCalibNEBULA; //must be called after parameter is constructed
  calibnebulahpc= new TArtCalibNEBULAHPC; //must be called after parameter is constructed
}

void TArtAnaLoopNEBULAHPC::Calculate()
{
  calibnebula->ClearData();
  calibnebulahpc->ClearData();

  calibnebulahpc->LoadRawData();
  calibnebulahpc->ReconstructData();

  //manual cut
  if(calibnebulahpc->fMultiplicity < 2) return;

  calibnebula->LoadRawData();
  calibnebula->ReconstructData();

  AddNEBULA();
  AddNEBULAHPC();
}

void TArtAnaLoopNEBULAHPC::Destruct()
{
  delete calibnebula;
  calibnebula = 0;
  delete calibnebulahpc;
  calibnebulahpc = 0;

  delete samuraiparameters;
  samuraiparameters = 0;
}

void TArtAnaLoopNEBULAHPC::AddNEBULA()
{
  using namespace Analyser;
  using namespace WNum::NEBULA;  

  Int_t NumNEBULAPla = calibnebula->GetNumNEBULAPla();
  for(int i=0; i<NumNEBULAPla; ++i){
    TArtNEBULAPla* pla = calibnebula->GetNEBULAPla(i);
    if(pla->GetDataState() < 0) continue;
    if(pla->GetIHitMin() < 4) continue;

    Int_t id    = pla->GetID();

    Add(NEBULA, ID,       id, id);
    Add(NEBULA, Layer,    id, pla->GetLayer());
    Add(NEBULA, SubLayer, id, pla->GetSubLayer());
    Add(NEBULA, DetPosX,  id, pla->GetDetPos(0));
    Add(NEBULA, DetPosY,  id, pla->GetDetPos(1));
    Add(NEBULA, DetPosZ,  id, pla->GetDetPos(2));

    Add(NEBULA, QURaw,    id, pla->GetQURaw());
    Add(NEBULA, QDRaw,    id, pla->GetQDRaw());
    Add(NEBULA, QUPed,    id, pla->GetQUPed());
    Add(NEBULA, QDPed,    id, pla->GetQDPed());
    Add(NEBULA, QUCal,    id, pla->GetQUCal());
    Add(NEBULA, QDCal,    id, pla->GetQDCal());
    Add(NEBULA, QAvePed,  id, pla->GetQAvePed());
    Add(NEBULA, QAveCal,  id, pla->GetQAveCal());
    Add(NEBULA, LogQPed,  id, pla->GetLogQPed());
    Add(NEBULA, LogQCal,  id, pla->GetLogQCal());

    Add(NEBULA, TURaw,    id, pla->GetTURaw());
    Add(NEBULA, TDRaw,    id, pla->GetTDRaw());
    Add(NEBULA, TUCal,    id, pla->GetTUCal());
    Add(NEBULA, TDCal,    id, pla->GetTDCal());
    Add(NEBULA, TUSlw,    id, pla->GetTUSlw());
    Add(NEBULA, TDSlw,    id, pla->GetTDSlw());
    Add(NEBULA, DTRaw,    id, pla->GetDTRaw());
    Add(NEBULA, DTCal,    id, pla->GetDTCal());
    Add(NEBULA, DTSlw,    id, pla->GetDTSlw());
    Add(NEBULA, TAveRaw,  id, pla->GetTAveRaw());
    Add(NEBULA, TAveCal,  id, pla->GetTAveCal());
    Add(NEBULA, TAveSlw,  id, pla->GetTAveSlw());

    Add(NEBULA, PosCal,   id, pla->GetPosCal());
    Add(NEBULA, PosSlw,   id, pla->GetPosSlw());
    Add(NEBULA, PosX,     id, pla->GetPos(0));
    Add(NEBULA, PosY,     id, pla->GetPos(1));
    Add(NEBULA, PosZ,     id, pla->GetPos(2));
  }
}

void TArtAnaLoopNEBULAHPC::AddNEBULAHPC()
{
  using namespace Analyser;
  using namespace WNum::NEBULAHPC;  

  Int_t NumNEBULAHPC = calibnebulahpc->GetNumNEBULAHPC();
  for(int i=0; i<NumNEBULAHPC; ++i){
    TArtNEBULAHPC* hpc = calibnebulahpc->GetNEBULAHPC(i);
    if(hpc->GetDataState() < 0) continue;

    Int_t id    = hpc->GetID();

    Add(NEBULAHPC, ID,       id, id);
    Add(NEBULAHPC, Layer,    id, hpc->GetLayer());
    Add(NEBULAHPC, SubLayer, id, hpc->GetSubLayer());
    Add(NEBULAHPC, DetPosX,  id, hpc->GetDetPos(0));
    Add(NEBULAHPC, DetPosY,  id, hpc->GetDetPos(1));
    Add(NEBULAHPC, DetPosZ,  id, hpc->GetDetPos(2));

    Add(NEBULAHPC, TRaw,     id, hpc->GetTRaw());
    Add(NEBULAHPC, TCal,     id, hpc->GetTCal());
  }
}

const char* TArtAnaLoopNEBULAHPC::ClassName() const
{
  return "TArtAnaLoopNEBULAHPC";
}
