
#include "TArtAnaLoopExample.hh"

#include <iostream>

#include "TArtAnaFile.hh"
#include "TArtAnaClock.hh"

#include "TArtSAMURAIParameters.hh"
#include "TArtCalibNEBULA.hh"
#include "TArtNEBULAPla.hh"
#include "TArtNEBULAPlaPara.hh"

#include "EAnalyser.hh"
#include "EWNum.hh"

TArtAnaLoopExample::TArtAnaLoopExample()
  : TArtAnaLoop(), samuraiparameters(0), calibnebula(0)
{;}

TArtAnaLoopExample::~TArtAnaLoopExample()
{
  Destruct();
}

void TArtAnaLoopExample::Construct()
{
  samuraiparameters = new TArtSAMURAIParameters;
  samuraiparameters->LoadParameter("db/NEBULA.xml");
  calibnebula= new TArtCalibNEBULA; //must be called after parameter is constructed
}

void TArtAnaLoopExample::Calculate()
{
  calibnebula->ClearData();

  calibnebula->LoadRawData();
  calibnebula->ReconstructData();

  AddNEBULA();
}

void TArtAnaLoopExample::Destruct()
{
  delete calibnebula;
  calibnebula = 0;

  delete samuraiparameters;
  samuraiparameters = 0;
}

void TArtAnaLoopExample::AddNEBULA()
{
  using namespace Analyser;
  using namespace WNum::NEBULA;  

  Int_t NumNEBULAPla = calibnebula->GetNumNEBULAPla();
  for(int i=0; i<NumNEBULAPla; ++i){
    TArtNEBULAPla* pla = calibnebula->GetNEBULAPla(i);
    if(pla->GetDataState() < 0) continue;
    if(pla->GetIHitMin() < 4) continue;

    Int_t id    = pla->GetID();

    Add(NEBULA, ID,       id, id);
    Add(NEBULA, Layer,    id, pla->GetLayer());
    Add(NEBULA, SubLayer, id, pla->GetSubLayer());
    Add(NEBULA, DetPosX,  id, pla->GetDetPos(0));
    Add(NEBULA, DetPosY,  id, pla->GetDetPos(1));
    Add(NEBULA, DetPosZ,  id, pla->GetDetPos(2));

    Add(NEBULA, QURaw,    id, pla->GetQURaw());
    Add(NEBULA, QDRaw,    id, pla->GetQDRaw());
    Add(NEBULA, QUPed,    id, pla->GetQUPed());
    Add(NEBULA, QDPed,    id, pla->GetQDPed());
    Add(NEBULA, QUCal,    id, pla->GetQUCal());
    Add(NEBULA, QDCal,    id, pla->GetQDCal());
    Add(NEBULA, QAvePed,  id, pla->GetQAvePed());
    Add(NEBULA, QAveCal,  id, pla->GetQAveCal());
    Add(NEBULA, LogQPed,  id, pla->GetLogQPed());
    Add(NEBULA, LogQCal,  id, pla->GetLogQCal());

    Add(NEBULA, TURaw,    id, pla->GetTURaw());
    Add(NEBULA, TDRaw,    id, pla->GetTDRaw());
    Add(NEBULA, TUCal,    id, pla->GetTUCal());
    Add(NEBULA, TDCal,    id, pla->GetTDCal());
    Add(NEBULA, TUSlw,    id, pla->GetTUSlw());
    Add(NEBULA, TDSlw,    id, pla->GetTDSlw());
    Add(NEBULA, DTRaw,    id, pla->GetDTRaw());
    Add(NEBULA, DTCal,    id, pla->GetDTCal());
    Add(NEBULA, DTSlw,    id, pla->GetDTSlw());
    Add(NEBULA, TAveRaw,  id, pla->GetTAveRaw());
    Add(NEBULA, TAveCal,  id, pla->GetTAveCal());
    Add(NEBULA, TAveSlw,  id, pla->GetTAveSlw());

    Add(NEBULA, PosCal,   id, pla->GetPosCal());
    Add(NEBULA, PosSlw,   id, pla->GetPosSlw());
    Add(NEBULA, PosX,     id, pla->GetPos(0));
    Add(NEBULA, PosY,     id, pla->GetPos(1));
    Add(NEBULA, PosZ,     id, pla->GetPos(2));
  }
}

const char* TArtAnaLoopExample::ClassName() const
{
  return "TArtAnaLoopExample";
}
