// BigRIPS/ZeroDegree calibration class
// 

#ifndef _TARTCALIBPID_H_
#define _TARTCALIBPID_H_

#include <TNamed.h>
#include <TClonesArray.h>

class TArtRawSegmentObject;
class TArtBigRIPSParameters;
class TArtCalibPPAC;
class TArtCalibPlastic;
class TArtCalibIC;
class TArtCalibFocalPlane;
class TArtStoreManager;

class TArtCalibPID  {

 public:

  TArtCalibPID();
  virtual ~TArtCalibPID();

  void LoadData();
  void LoadData(TArtRawSegmentObject *seg);
  void ClearData();
  void ReconstructData();

  bool IsReconstructed(){ return fReconstructed; }

  // function to access data container
  TClonesArray    * GetPPACArray();
  TClonesArray    * GetPlasticArray();
  TClonesArray    * GetICArray();
  TClonesArray    * GetFocalPlaneArray();

  // function to access data calibration class
  TArtCalibPPAC   * GetCalibPPAC(){return fCalibPPAC;}
  TArtCalibPlastic* GetCalibPlastic(){return fCalibPlastic;}
  TArtCalibIC     * GetCalibIC(){return fCalibIC;}
  TArtCalibFocalPlane * GetCalibFocalPlane(){return fCalibFocalPlane;}

 private:

  bool fDataLoaded;
  bool fReconstructed;

  // pointer to subsystem objects used for PID
  TArtCalibPPAC    * fCalibPPAC;
  TArtCalibIC      * fCalibIC;
  TArtCalibPlastic * fCalibPlastic;
  TArtCalibFocalPlane * fCalibFocalPlane;

  TArtBigRIPSParameters * setup;
  TArtStoreManager * sman;

  ClassDef(TArtCalibPID,1);

};

#endif
