#include "TArtRecoTOF.h" 
#include "TArtPlastic.h"
#include "TArtPlasticPara.h"
#include "TArtTOF.h"
#include "TArtBigRIPSParameters.h"
#include "TArtCore.h" 
#include "TArtStoreManager.h"

#include <TROOT.h>
#include <TString.h>
#include <TClonesArray.h>

//__________________________________________________________
TArtRecoTOF::TArtRecoTOF()
  :fReconstructed(false)
{
  TArtCore::Info(__FILE__,"Creating the TOF objects...");
  sman = TArtStoreManager::Instance();
  setup = (TArtBigRIPSParameters*)sman->FindParameters("BigRIPSParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: BigRIPSParameters");
    throw;
  }

  fTOFArray = new TClonesArray("TArtTOF",10);
  fTOFArray->SetName("BigRIPSTOF");
  fTOFArray->SetOwner();
  sman->AddDataContainer(fTOFArray);
}

//__________________________________________________________
TArtRecoTOF::~TArtRecoTOF()  {
  ClearData();
}

//__________________________________________________________
TArtTOF * TArtRecoTOF::DefineNewTOF(char *uplname, char *dplname, Double_t offset){

  char name[64];
  Int_t ntof=GetNumTOF();
  new ((*fTOFArray)[ntof]) TArtTOF();
  TArtTOF * tof = (TArtTOF *)fTOFArray->At(ntof);

  sprintf(name,"TOF%sto%s",uplname,dplname);
  TArtCore::Info(__FILE__,"define %s",name);
  tof->SetDetectorName(name);
  tof->SetUpstreamPlaName(uplname);
  tof->SetDownstreamPlaName(dplname);
  tof->SetTimeOffset(offset);

  TString uplstr = uplname;
  TArtPlasticPara * upara = setup->GetPlasticPara(&uplstr);
  TString dplstr = dplname;
  TArtPlasticPara * dpara = setup->GetPlasticPara(&dplstr);
  tof->SetUpstreamPlaFpl(upara->GetFpl());
  tof->SetDownstreamPlaFpl(dpara->GetFpl());

  return tof;

}

//__________________________________________________________
void TArtRecoTOF::ClearData()   {
  for(int i=0;i<GetNumTOF();i++)
    fTOFArray->At(i)->Clear();
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtRecoTOF::ReconstructData()   { // call after the raw data are loaded

  TClonesArray * pla_array = (TClonesArray *)sman->FindDataContainer("BigRIPSPlastic");
  Int_t num_pla = pla_array->GetEntries();

  //on the assumption of linear track, the track is reconstructed analytically.
  //X = a + b * Z;

  for(int i=0;i<GetNumTOF();i++){
    TArtPlastic * upstream_pla = NULL;
    TArtPlastic * downstream_pla = NULL;
    TArtTOF *tof = (TArtTOF *)fTOFArray->At(i);

    /////////////////////////////////////////////////////////////////////
    // find fpl objects
    bool FindAllObjects = true;

    for(int j=0;j<num_pla;j++){
      TArtPlastic *tmp_pla = (TArtPlastic *)pla_array->At(j);
      TString planame = *(tmp_pla->GetDetectorName());

      if(planame ==  *(tof->GetUpstreamPlaName()))
	upstream_pla = tmp_pla; 
      if(planame ==  *(tof->GetDownstreamPlaName()))
	downstream_pla = tmp_pla; 
    }

    if(NULL == upstream_pla){
      TArtCore::Info(__FILE__,"no upstream pla: %s - %s",tof->GetUpstreamPlaName()->Data(), tof->GetDownstreamPlaName()->Data());
      FindAllObjects = false;
    }
    if(NULL == downstream_pla){
      TArtCore::Info(__FILE__,"no downstream pla: %s - %s",tof->GetUpstreamPlaName()->Data(), tof->GetDownstreamPlaName()->Data());
      FindAllObjects = false;
    }

    if(!FindAllObjects) continue;
    /////////////////////////////////////////////////////////////////////
    // start to reconstruct 

    tof->SetLength(downstream_pla->GetZposition()-upstream_pla->GetZposition());
    tof->SetTOF(downstream_pla->GetTimeSlew() - upstream_pla->GetTimeSlew() + tof->GetTimeOffset());

  } 

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtTOF * TArtRecoTOF::GetTOF(int i) {
  return (TArtTOF *)fTOFArray->At(i);
}
//__________________________________________________________
Int_t TArtRecoTOF::GetNumTOF() {
  return fTOFArray->GetEntries();
}
//__________________________________________________________
TArtTOF * TArtRecoTOF::FindTOF(char * u_planame, char * d_planame){
  for(int i=0;i<GetNumTOF();i++)
    if(((TArtTOF*)fTOFArray->At(i))->GetUpstreamPlaName()->CompareTo(u_planame) == 0 && ((TArtTOF*)fTOFArray->At(i))->GetDownstreamPlaName()->CompareTo(d_planame) == 0)
      return (TArtTOF*)fTOFArray->At(i);
  return NULL;
}
